/**
 * Differential equations for a double pendulum consisting of two
 * bars moving without friction. Angles phi1 and phi2 are measured from a downwart
 * pointing vertical direction. Angular velocities are denoted w1 and w2.
 * In each case subscript 1 represents upper bar and subscript 2 represents
 * lower bar.
 *
 * Author: Slavomir Tuleja, July 30, 2003
 */
package org.opensourcephysics.tuleja.doublependulum3d;

import org.opensourcephysics.display.*;

public class DoublePendulum
    implements DoublePendulumODE {
  private double[] state = new double[] {
      0.0, 0.0, 0.0, 0.0, 0.0};
  //...pociatocny stav v tvare {phi1,phi2,w1,w2,t}
  private double L1 = 1;
  private double L2 = 2;
  private double g = 9.8;

  public double[] getState() {
    return state;
  }

  public void getRate(double[] state, double[] rate) {
    double phi1, phi2, phi1D, phi2D;
    phi1 = state[0];
    phi2 = state[1];
    phi1D = state[2];
    phi2D = state[3];
    rate[0] = phi1D;
    rate[1] = phi2D;
    rate[2] = - (3 *
                 (g *
                  (4 * L1 * Math.sin(phi1) + 5 * L2 * Math.sin(phi1) +
                   3 * L2 * Math.sin(phi1 - 2 * phi2)) +
                  3 * L1 * L2 * Math.sin(2 * (phi1 - phi2)) * phi1D * phi1D +
                  4 * L2 * L2 * Math.sin(phi1 - phi2) * phi2D * phi2D)) /
        (L1 * (8 * L1 + 15 * L2 - 9 * L2 * Math.cos(2 * (phi1 - phi2))));
    rate[3] = - (3 *
                 (g *
                  (3 * (L1 + 2 * L2) * Math.sin(2 * phi1 - phi2) -
                   (L1 + 6 * L2) * Math.sin(phi2)) +
                  4 * L1 * (L1 + 3 * L2) * Math.sin(phi1 - phi2) * phi1D *
                  phi1D +
                  3 * L2 * L2 * Math.sin(2 * (phi1 - phi2)) * phi2D * phi2D)) /
        (L2 * ( -8 * L1 - 15 * L2 + 9 * L2 * Math.cos(2 * (phi1 - phi2))));
    rate[4] = 1;
  }

  public void setPhi1(double phi1) {
    state[0] = phi1;
  }

  public void setPhi2(double phi2) {
    state[1] = phi2;
  }

  public void setW1(double w1) {
    state[2] = w1;
  }

  public void setW2(double w2) {
    state[3] = w2;
  }

  public void setT(double t) {
    state[4] = t;
  }

  public void setL1(double L1) {
    this.L1 = L1;
  }

  public void setL2(double L2) {
    this.L2 = L2;
  }

  public double getPhi1() {
    return state[0];
  }

  public double getPhi2() {
    return state[1];
  }

  public double getW1() {
    return state[2];
  }

  public double getW2() {
    return state[3];
  }

  public double getT() {
    return state[4];
  }

  public double getL1() {
    return L1;
  }

  public double getL2() {
    return L2;
  }

}