package org.opensourcephysics.tuleja.feynmanplate;

import org.opensourcephysics.numerics.*;
import org.opensourcephysics.displayejs.*;
import java.awt.Color;
import java.awt.Font;

/**
 * <p>Title: Cuboid</p>
 * <p>Description: Subclass of RigidBody. It can draw itself and compute its motion.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * @author Alexander Tomori, Boris Gazovic, Slavomir Tuleja
 * @version 0.01
 */
public class Cuboid extends RigidBody{
  InteractiveSurface cube;
  InteractiveSurface srfB1, srfB2, srf1, srf2, srf3, srf4;//walls of the cube

  int n=10;//number of small rectangles on the surface in each direction

  double[][][] dataB1=new double[n][n][3];//lower base of the cube
  double[][][] dataB2=new double[n][n][3];//upper base of the cube
  double[][][] data1=new double[n][n][3];//1st xz wall
  double[][][] data2=new double[n][n][3];//1st yz wall
  double[][][] data3=new double[n][n][3];//2nd xz wall
  double[][][] data4=new double[n][n][3];//2nd yz wall


  /**
   * Constructor of the cuboid.
   */
  public Cuboid(){
    super(0.130,0.065);

    double a=0.5;
    double v=a*Math.sqrt(3);

    srfB1=new InteractiveSurface();//base
    srfB2=new InteractiveSurface();//upper base
    srf1=new InteractiveSurface();//surfaces belonging to the walls of the cube
    srf2=new InteractiveSurface();
    srf3=new InteractiveSurface();
    srf4=new InteractiveSurface();

    for(int i=0; i<n; i++){
      for(int j=0; j<n; j++){
        //lower base
        dataB1[i][j][0]=a* i/(n-1) -a/2.0;
        dataB1[i][j][1]=a* j/(n-1) -a/2.0;
        dataB1[i][j][2]=-v/2.0;

        //upper base
        dataB2[i][j][0]=a* i/(n-1) -a/2.0;
        dataB2[i][j][1]=a* j/(n-1) -a/2.0;
        dataB2[i][j][2]=v/2.0;

        //1st xz wall
        data1[i][j][0]=a* i/(n-1) -a/2.0;
        data1[i][j][1]=-a/2.0;
        data1[i][j][2]=v* j/(n-1) -v/2.0;

        //2nd xz wall
        data3[i][j][0]=a* i/(n-1) -a/2.0;
        data3[i][j][1]=a/2.0;
        data3[i][j][2]=v* j/(n-1) -v/2.0;

        //1st yz wall
        data2[i][j][0]=a/2.0;
        data2[i][j][1]=a* i/(n-1) -a/2.0;
        data2[i][j][2]=v* j/(n-1) -v/2.0;

        //2nd yz wall
        data4[i][j][0]=-a/2.0;
        data4[i][j][1]=a* i/(n-1) -a/2.0;
        data4[i][j][2]=v* j/(n-1) -v/2.0;
      }
    }


    srfB1.setData(dataB1);
    srfB2.setData(dataB2);
    srf1.setData(data1);
    srf2.setData(data2);
    srf3.setData(data3);
    srf4.setData(data4);

    //styles and colors
    srfB1.getStyle().setEdgeColor(new Color(53, 97, 255));
    srfB1.setColorRegions(new double[]{}, new Color[]{new Color(198, 222, 255)});
    srfB2.getStyle().setEdgeColor(new Color(53, 97, 255));
    srfB2.setColorRegions(new double[]{}, new Color[]{new Color(198, 222, 255)});
    srf1.getStyle().setEdgeColor(new Color(53, 97, 255));
    srf1.setColorRegions(new double[]{}, new Color[]{new Color(198, 222, 255)});
    srf2.getStyle().setEdgeColor(new Color(53, 97, 255));
    srf2.setColorRegions(new double[]{}, new Color[]{new Color(198, 222, 255)});
    srf3.getStyle().setEdgeColor(new Color(53, 97, 255));
    srf3.setColorRegions(new double[]{}, new Color[]{new Color(198, 222, 255)});
    srf4.getStyle().setEdgeColor(new Color(53, 97, 255));
    srf4.setColorRegions(new double[]{}, new Color[]{new Color(198, 222, 255)});

    srf1.setOrigin(0,0,0,false);
    srf2.setOrigin(0,0,0,false);
    srf3.setOrigin(0,0,0,false);
    srf4.setOrigin(0,0,0,false);
    srfB1.setOrigin(0,0,0,false);
    srfB2.setOrigin(0,0,0,false);


    add(srfB1);
    add(srfB2);
    add(srf1);
    add(srf2);
    add(srf3);
    add(srf4);

    setVisibility();
  }


  public void setVisibility(){
    super.setVisibilityExceptRigidBody();
    srf1.setVisible(showRigidBody);
    srf2.setVisible(showRigidBody);
    srf3.setVisible(showRigidBody);
    srf4.setVisible(showRigidBody);
    srfB1.setVisible(showRigidBody);
    srfB2.setVisible(showRigidBody);
  }

  public void transformBodyOnly(double[][] rm){
   srfB1.setData(transformOneFace(dataB1, rm));
   srfB2.setData(transformOneFace(dataB2, rm));
   srf1.setData(transformOneFace(data1, rm));
   srf2.setData(transformOneFace(data2, rm));
   srf3.setData(transformOneFace(data3, rm));
   srf4.setData(transformOneFace(data4, rm));
  }


  /**
   * Transforms a face of the cube.
   * @param data data of the face-surface
   * @param rm rotation matrix used to transformation in space
   * @return transformed data of surface
   */
  public double[][][] transformOneFace(double[][][] data, double[][] rm){
    double[][][] dataTransformed=new double[n][n][3];
    for (int i = 0; i < n; i++) {
     for (int j = 0; j < n; j++) {

       double x = data[i][j][0];
       double y = data[i][j][1];
       double z = data[i][j][2];


       double xn = rm[0][0] * x + rm[0][1] * y + rm[0][2] * z;
       double yn = rm[1][0] * x + rm[1][1] * y + rm[1][2] * z;
       double zn = rm[2][0] * x + rm[2][1] * y + rm[2][2] * z;

       dataTransformed[i][j][0]= xn;
       dataTransformed[i][j][1]= yn;
       dataTransformed[i][j][2]= zn;
     }
   }
   return dataTransformed;
  }



}
